@@ -5,6 +5,7 @@
 #define MVM_P6INT_C_TYPE_LONGLONG  -5
 #define MVM_P6INT_C_TYPE_SIZE_T    -6
 #define MVM_P6INT_C_TYPE_BOOL      -7
+#define MVM_P6INT_C_TYPE_ATOMIC    -8
 
 /* Representation used by P6 native ints. */
 struct MVMP6intBody {
@@ -34,3 +35,6 @@ struct MVMP6intREPRData {
 
 /* Function for REPR setup. */
 const MVMREPROps * MVMP6int_initialize(MVMThreadContext *tc);
+
+void MVMP6int_set_int(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 value);
+MVMint64 MVMP6int_get_int(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data);
