@@ -20,5 +20,7 @@ struct MVMReentrantMutex {
 const MVMREPROps * MVMReentrantMutex_initialize(MVMThreadContext *tc);
 
 /* Lock and unlock functions. */
+void MVM_reentrantmutex_lock_checked(MVMThreadContext *tc, MVMObject *lock);
 void MVM_reentrantmutex_lock(MVMThreadContext *tc, MVMReentrantMutex *rm);
+void MVM_reentrantmutex_unlock_checked(MVMThreadContext *tc, MVMObject *lock);
 void MVM_reentrantmutex_unlock(MVMThreadContext *tc, MVMReentrantMutex *rm);
