@@ -1,6 +1,6 @@
 /* Callsite argument flags. */
-#define MVM_CALLSITE_ARG_TYPE_MASK       15
-#define MVM_CALLSITE_ARG_NAMED_FLAT_MASK 31
+#define MVM_CALLSITE_ARG_TYPE_MASK       143
+#define MVM_CALLSITE_ARG_NAMED_FLAT_MASK 159
 typedef enum {
     /* Argument is an object. */
     MVM_CALLSITE_ARG_OBJ = 1,
@@ -23,14 +23,15 @@ typedef enum {
     /* Argument is flattened. What this means is up to the target. */
     MVM_CALLSITE_ARG_FLAT = 64,
 
-    /* Argument is flattened and named. */
-    MVM_CALLSITE_ARG_FLAT_NAMED = 128
+    /* Argument is a native integer, unsigned. */
+    MVM_CALLSITE_ARG_UINT = 128,
 } MVMCallsiteFlags;
 
 /* Callsites that are used within the VM. */
 typedef enum {
     MVM_CALLSITE_ID_ZERO_ARITY,
     MVM_CALLSITE_ID_OBJ,
+    MVM_CALLSITE_ID_STR,
     MVM_CALLSITE_ID_INT,
     MVM_CALLSITE_ID_OBJ_OBJ,
     MVM_CALLSITE_ID_OBJ_INT,
@@ -39,6 +40,7 @@ typedef enum {
     MVM_CALLSITE_ID_INT_INT,
     MVM_CALLSITE_ID_OBJ_OBJ_STR,
     MVM_CALLSITE_ID_OBJ_OBJ_OBJ,
+    MVM_CALLSITE_ID_OBJ_UINT,
 } MVMCommonCallsiteID;
 
 /* A callsite entry is just one of the above flags. */
@@ -87,12 +89,11 @@ struct MVMCallsite {
 /* Interned callsites data structure. */
 struct MVMCallsiteInterns {
     /* 2-level array of pointers to callsites, first indexed by the arity,
-     * then callsite of that arity. Allocated via the FSA and freed at a
-     * safepoint if we grow beyond it. */
+     * then callsite of that arity. Freed at a safepoint if we grow beyond it. */
     MVMCallsite ***by_arity;
 
-    /* Number of callsites we have interned by each arity. Also allocated
-     * using the FSA and freed at a safepoint on growth. */
+    /* Number of callsites we have interned by each arity. Also freed at a safepoint
+     * on growth. */
     MVMuint32 *num_by_arity;
 
     /* The maximum interned arity so far (the element count of the above two
@@ -132,7 +133,7 @@ MVM_STATIC_INLINE MVMuint16 MVM_callsite_num_nameds(MVMThreadContext *tc, const
     MVMuint16 i = cs->num_pos;
     MVMuint16 nameds = 0;
     while (i < cs->flag_count) {
-        if (!(cs->arg_flags[i] & MVM_CALLSITE_ARG_FLAT_NAMED))
+        if (!(cs->arg_flags[i] & MVM_CALLSITE_ARG_FLAT))
             nameds++;
         i++;
     }
@@ -148,6 +149,8 @@ MVM_STATIC_INLINE const char * MVM_callsite_arg_type_name(MVMCallsiteFlags f) {
             return "str";
         case MVM_CALLSITE_ARG_INT:
             return "int";
+        case MVM_CALLSITE_ARG_UINT:
+            return "uint";
         case MVM_CALLSITE_ARG_NUM:
             return "num";
         default:
