@@ -28,10 +28,18 @@
 #define MVM_NATIVECALL_ARG_NO_FREE_STR     0
 #define MVM_NATIVECALL_ARG_FREE_STR        1
 #define MVM_NATIVECALL_ARG_FREE_STR_MASK   1
+/* Flag for whether we need to refresh a CArray after passing or not. */
+#define MVM_NATIVECALL_ARG_NO_REFRESH      0
+#define MVM_NATIVECALL_ARG_REFRESH         1
+#define MVM_NATIVECALL_ARG_REFRESH_MASK    1
 #define MVM_NATIVECALL_ARG_NO_RW           0
 #define MVM_NATIVECALL_ARG_RW              256
 #define MVM_NATIVECALL_ARG_RW_MASK         256
 
+#define MVM_NATIVECALL_UNMARSHAL_KIND_GENERIC     -1
+#define MVM_NATIVECALL_UNMARSHAL_KIND_RETURN      -2
+#define MVM_NATIVECALL_UNMARSHAL_KIND_NATIVECAST  -3
+
 /* Native callback entry. Hung off MVMNativeCallbackCacheHead, which is
  * a hash owned by the ThreadContext. All MVMNativeCallbacks in a linked
  * list have the same cuid, which is the key to the CacheHead hash.
@@ -40,8 +48,8 @@ struct MVMNativeCallback {
     /* The dyncall/libffi callback object. */
     void *cb;
 
-    /* The routine that we will call. */
-    MVMObject *target;
+    /* The code handle that we will call. */
+    MVMCode *target;
 
     /* The VM instance. */
     MVMInstance *instance;
@@ -75,10 +83,8 @@ struct MVMNativeCallback {
  * closures are taken and need to be differentiated.
  */
 struct MVMNativeCallbackCacheHead {
+    struct MVMStrHashHandle hash_handle;
     MVMNativeCallback *head;
-
-    /* The uthash hash handle inline struct. */
-    UT_hash_handle hash_handle;
 };
 
 /* Functions for working with native callsites. */
@@ -86,9 +92,12 @@ MVMNativeCallBody * MVM_nativecall_get_nc_body(MVMThreadContext *tc, MVMObject *
 MVMint16 MVM_nativecall_get_arg_type(MVMThreadContext *tc, MVMObject *info, MVMint16 is_return);
 MVMint8 MVM_nativecall_build(MVMThreadContext *tc, MVMObject *site, MVMString *lib,
     MVMString *sym, MVMString *conv, MVMObject *arg_spec, MVMObject *ret_spec);
+void MVM_nativecall_setup(MVMThreadContext *tc, MVMNativeCallBody *body, unsigned int interval_id);
+void MVM_nativecall_restore_library(MVMThreadContext *tc, MVMNativeCallBody *body, MVMObject *root);
 MVMObject * MVM_nativecall_invoke(MVMThreadContext *tc, MVMObject *res_type,
     MVMObject *site, MVMObject *args);
-void MVM_nativecall_invoke_jit(MVMThreadContext *tc, MVMObject *site);
+void MVM_nativecall_dispatch(MVMThreadContext *tc, MVMObject *res_type,
+    MVMObject *site, MVMArgs args);
 MVMObject * MVM_nativecall_global(MVMThreadContext *tc, MVMString *lib, MVMString *sym,
     MVMObject *target_spec, MVMObject *target_type);
 MVMObject * MVM_nativecall_cast(MVMThreadContext *tc, MVMObject *target_spec,
@@ -120,14 +129,27 @@ unsigned long long  MVM_nativecall_unmarshal_ulonglong(MVMThreadContext *tc, MVM
 float               MVM_nativecall_unmarshal_float(MVMThreadContext *tc, MVMObject *value);
 double              MVM_nativecall_unmarshal_double(MVMThreadContext *tc, MVMObject *value);
 
-char * MVM_nativecall_unmarshal_string(MVMThreadContext *tc, MVMObject *value, MVMint16 type, MVMint16 *free);
-void * MVM_nativecall_unmarshal_cstruct(MVMThreadContext *tc, MVMObject *value);
-void * MVM_nativecall_unmarshal_cppstruct(MVMThreadContext *tc, MVMObject *value);
-void * MVM_nativecall_unmarshal_cpointer(MVMThreadContext *tc, MVMObject *value);
-void * MVM_nativecall_unmarshal_carray(MVMThreadContext *tc, MVMObject *value);
-void * MVM_nativecall_unmarshal_vmarray(MVMThreadContext *tc, MVMObject *value);
-void * MVM_nativecall_unmarshal_cunion(MVMThreadContext *tc, MVMObject *value);
-MVMThreadContext * MVM_nativecall_find_thread_context(MVMInstance *instance);
+char * MVM_nativecall_encode_string(MVMThreadContext *tc, MVMString *value_str, MVMint16 type, MVMint16 *free, MVMint16 unmarshal_kind, const MVMREPROps *repr);
+char * MVM_nativecall_unmarshal_string(MVMThreadContext *tc, MVMObject *value, MVMint16 type, MVMint16 *free, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_cstruct(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_cppstruct(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_cpointer(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_carray(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_vmarray(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+void * MVM_nativecall_unmarshal_cunion(MVMThreadContext *tc, MVMObject *value, MVMint16 unmarshal_kind);
+
+/* Locate the thread that a callback should be run on. */
+MVM_STATIC_INLINE MVMThreadContext * MVM_nativecall_find_thread_context(MVMInstance *instance) {
+    MVMThreadContext *tc = MVM_get_running_threads_context();
+
+    if (!tc)
+        MVM_panic(1, "native callback ran on thread (%"PRId64") unknown to MoarVM",
+                  MVM_platform_thread_id());
+
+    return tc;
+}
+
+
 MVMJitGraph *MVM_nativecall_jit_graph_for_caller_code(
     MVMThreadContext   *tc,
     MVMSpeshGraph      *sg,
