@@ -7,20 +7,29 @@
 #include <math.h>
 
 #if defined(INFINITY) && !defined(_AIX)
-static const MVMnum64 MVM_NUM_POSINF =  INFINITY;
+#  ifdef _MSC_VER
+#define MVM_NUM_POSINF  INFINITY
+#define MVM_NUM_NEGINF -INFINITY
+#  else
+static const MVMnum64 MVM_NUM_POSINF = INFINITY;
 static const MVMnum64 MVM_NUM_NEGINF = -INFINITY;
+#  endif
 #else
 #  ifdef _MSC_VER
 #define MVM_NUM_POSINF  (DBL_MAX+DBL_MAX)
 #define MVM_NUM_NEGINF -(DBL_MAX+DBL_MAX)
 #  else
-static const MVMnum64 MVM_NUM_POSINF =  1.0 / 0.0;
+static const MVMnum64 MVM_NUM_POSINF = 1.0 / 0.0;
 static const MVMnum64 MVM_NUM_NEGINF = -1.0 / 0.0;
 #  endif
 #endif
 
 #if defined(NAN) && !defined(_AIX)
+#  ifdef _MSC_VER
+#define MVM_NUM_NAN NAN
+#  else
 static const MVMnum64 MVM_NUM_NAN = NAN;
+#  endif
 #else
 #  ifdef _MSC_VER
 #define MVM_NUM_NAN (MVM_NUM_POSINF-MVM_NUM_POSINF)
